#pragma once
///////////////////////////////////////////////////////////////////////////////
// File: PerformanceCounters.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/PerfMonTools/PerformanceMonitorCountersBase.h"
#include "JetByteTools/PerfMonTools/ILoggablePerformanceMonitorCounters.h"
#include "JetByteTools/PerfMonTools/PerformanceCounterLogger.h"

#include "PerformanceDataSchema.h"

#include "JetByteTools/CoreTools/IMonitorThreadPool.h"
#include "JetByteTools/IOTools/IMonitorBufferAllocation.h"

#include "JetByteTools/SocketTools/IMonitorSocketAllocation.h"

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools
{
   namespace Win32
   {
      class CSecurityAttributes;
   }
}

///////////////////////////////////////////////////////////////////////////////
// CPerformanceCounters
///////////////////////////////////////////////////////////////////////////////

class CPerformanceCounters :
   private JetByteTools::PerfMon::ILoggablePerformanceMonitorCounters,
   public JetByteTools::IO::IMonitorBufferAllocation,
   public JetByteTools::Core::IMonitorThreadPool,
   public JetByteTools::Socket::IMonitorSocketAllocation
{
   public:

      CPerformanceCounters(
         bool createCounters,
         JetByteTools::Win32::CSecurityAttributes &sa,
         const JetByteTools::Core::_tstring &logName,
         const JetByteTools::Core::_tstring &outputFileName);

      CPerformanceCounters(
         const CPerformanceCounters &rhs) = delete;

      ~CPerformanceCounters();

      CPerformanceCounters &operator=(
         const CPerformanceCounters &rhs) = delete;

      bool LogCountersWithOptionalAdditionalCounters(
         const JetByteTools::Core::_tstring &templateFile) const;

      bool LogCountersWithOptionalAdditionalCounters(
         const JetByteTools::Core::_tstring &templateFile,
         const JetByteTools::Core::_tstring &parameters) const;

      static bool InstallCounters();

      static bool RemoveCounters();

      void BytesProcessed(
         long bytes);

      void OnConnectionStarted();
      void OnConnectionComplete();

   private :

      // Implement ILoggablePerformanceMonitorCounters

      bool CountersAreActive() const override;

      const CPerformanceDataSchema &GetSchema() const override;

      bool GetInstanceNamesForObject(
         const JetByteTools::Core::_tstring &objectName,
         JetByteTools::Core::StringVector &instanceNames) const override;

      // Implement IMonitorBufferAllocation

      void OnBufferCreated() override;
      void OnBufferAllocated(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;
      void OnBufferAllocationContention() override;
      void OnBufferReleaseContention() override;
      void OnBufferReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;
      void OnBufferDestroyed() override;

      // Implement IMonitorThreadPool

      void OnThreadPoolError(
         const JetByteTools::Core::_tstring &message) override;
      void OnThreadPoolThreadCreated() override;
      void OnThreadPoolThreadStarted() override {}
      void OnThreadPoolDispatch() override;
      void OnThreadPoolWorkItemsDequeued(
         const DWORD /*numWorkItems*/) override {}
      void OnThreadPoolProcessingStarted(
         bool dataAvailable) override;
      void OnThreadPoolProcessingStopped(
         bool dataAvailable) override;
      void OnThreadPoolThreadStopped() override {}
      void OnThreadPoolThreadDestroyed() override;

      // Implement IMonitorSocketAllocation

      void OnSocketCreated() override;
      void OnSocketAttached(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;
      void OnSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;
      void OnSocketDestroyed() override;

      CPerformanceDataSchema m_schema;

      JetByteTools::PerfMon::CPerformanceMonitorCountersBase *m_pCounters;

      const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::ObjectId m_object;

      const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::CounterId m_bytesProcessedCounter;
      const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::CounterId m_totalBuffersCounter;
      const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::CounterId m_buffersInUseCounter;
      const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::CounterId m_connectionsActiveCounter;
      const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::CounterId m_totalSocketsCounter;
      const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::CounterId m_socketsInUseCounter;
      const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::CounterId m_ioThreadsActiveCounter;
      const JetByteTools::PerfMon::CPerformanceMonitorCountersBase::CounterId m_ioThreadsProcessingCounter;

      JetByteTools::PerfMon::CPerformanceCounterLogger m_logger;
};

///////////////////////////////////////////////////////////////////////////////
// End of file: PerformanceCounters.h
///////////////////////////////////////////////////////////////////////////////
