#pragma once
///////////////////////////////////////////////////////////////////////////////
// File: PerformanceDataCollector.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/PerfMonTools/PerformanceDataCollector.h"
#include "JetByteTools/PerfMonTools/IPerformanceDataBlockFactory.h"

#include "PerformanceDataSchema.h"

#include "JetByteTools/CoreTools/tstring.h"

#include "JetByteTools/Win32Tools/SecurityAttributes.h"
#include "JetByteTools/Win32Tools/SecurityDescriptorAllowAll.h"
#include "JetByteTools/Win32Tools/GlobalName.h"

///////////////////////////////////////////////////////////////////////////////
// CPerformanceDataCollector
///////////////////////////////////////////////////////////////////////////////

class CPerformanceDataCollector :
   public JetByteTools::PerfMon::ICollectPerformanceData,
   private JetByteTools::PerfMon::IPerformanceDataBlockFactory
{
   public:

      CPerformanceDataCollector(
         const JetByteTools::Core::_tstring &applicationName);

      CPerformanceDataCollector(
         const CPerformanceDataCollector &rhs) = delete;

      CPerformanceDataCollector &operator=(
         const CPerformanceDataCollector &rhs) = delete;

      // Implement ICollectPerformanceData

      void Open(
         const wchar_t *pDevNames) override;

      void Close() override;

      bool CollectW(
         const wchar_t *pszValueName, 
         BYTE **ppData, 
         DWORD *pcbTotalBytes, 
         DWORD *pNumObjectTypes) override;

   private :

      // Implement IPerformanceDataBlockFactory

      JetByteTools::PerfMon::IPerformanceDataBlock &AllocatePerformanceDataBlock(
         const JetByteTools::PerfMon::CPerformanceDataSchema &schema,
         DWORD firstCounterIndex) override;

      void ReleasePerformanceDataBlock(
         JetByteTools::PerfMon::IPerformanceDataBlock &dataBlock)  override;

      const JetByteTools::Win32::CGlobalName m_name;

      CPerformanceDataSchema m_schema;

      JetByteTools::Win32::CSecurityDescriptorAllowAll m_sd;

      JetByteTools::Win32::CSecurityAttributes m_sa;

      JetByteTools::PerfMon::CPerformanceDataCollector m_collector;
};

///////////////////////////////////////////////////////////////////////////////
// End of file: PerformanceDataCollector.h
///////////////////////////////////////////////////////////////////////////////
