///////////////////////////////////////////////////////////////////////////////
// File: PerformanceDataSchema.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2007 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "PerformanceDataSchema.h"

#pragma hdrstop

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::_tstring;

using JetByteTools::PerfMon::DetailLevelNovice;
using JetByteTools::PerfMon::CounterTypeCounter32;
using JetByteTools::PerfMon::CounterTypeCounter64;
using JetByteTools::PerfMon::CounterTypePerSec32;

///////////////////////////////////////////////////////////////////////////////
// Static constants
///////////////////////////////////////////////////////////////////////////////

const _tstring CPerformanceDataSchema::s_objectName = _T("JetByte Echo Server Service");

const _tstring CPerformanceDataSchema::s_bytesProcessedCounter = _T("Bytes");

const _tstring CPerformanceDataSchema::s_bytesPerSecondCounter = _T("Bytes/sec");

const _tstring CPerformanceDataSchema::s_totalBuffersCounter = _T("IO Buffers Total");

const _tstring CPerformanceDataSchema::s_buffersInUseCounter = _T("IO Buffers In Use");

const _tstring CPerformanceDataSchema::s_connectionsActiveCounter = _T("Connections Active");

const _tstring CPerformanceDataSchema::s_totalSocketsCounter = _T("Sockets Total");

const _tstring CPerformanceDataSchema::s_socketsInUseCounter = _T("Sockets In Use");

const _tstring CPerformanceDataSchema::s_ioThreadsActiveCounter = _T("IO Threads Active");

const _tstring CPerformanceDataSchema::s_ioThreadsProcessingCounter = _T("IO Threads Processing");

///////////////////////////////////////////////////////////////////////////////
// CPerformanceDataSchema
///////////////////////////////////////////////////////////////////////////////

CPerformanceDataSchema::CPerformanceDataSchema()
{
   Object &exeObject = AddObject(
      s_objectName,
      _T("The \"JetByte Echo Server Service\" object type defines ")
      _T("counters that apply to the Echo Server Service process as ")
      _T("a whole. "),
      DetailLevelNovice);

   exeObject.AddCounter(
      s_bytesProcessedCounter,
      _T("The total number of bytes processed"),
      DetailLevelNovice,
      -6,
      CounterTypeCounter64);

   exeObject.SetDefaultCounter();

   exeObject.AddLinkedPerSecondCounter(
      s_bytesPerSecondCounter,
      _T("The number of bytes processed per second"),
      DetailLevelNovice,
      -4);

   exeObject.AddCounter(
      s_totalBuffersCounter,
      _T("The total number of IO buffers available"),
      DetailLevelNovice,
      0,
      CounterTypeCounter32);

   exeObject.AddCounter(
      s_buffersInUseCounter,
      _T("The number of IO buffers currently in use"),
      DetailLevelNovice,
      0,
      CounterTypeCounter32);

   exeObject.AddCounter(
      s_connectionsActiveCounter,
      _T("The number of connected sockets"),
      DetailLevelNovice,
      0,
      CounterTypeCounter32);

   exeObject.AddCounter(
      s_totalSocketsCounter,
      _T("The total number of sockets in the socket pool"),
      DetailLevelNovice,
      0,
      CounterTypeCounter32);

   exeObject.AddCounter(
      s_socketsInUseCounter,
      _T("The total number of sockets currently in use"),
      DetailLevelNovice,
      0,
      CounterTypeCounter32);

   exeObject.AddCounter(
      s_ioThreadsActiveCounter,
      _T("The total number of I/O threads in the thread pool"),
      DetailLevelNovice,
      0,
      CounterTypeCounter32);

   exeObject.AddCounter(
      s_ioThreadsProcessingCounter,
      _T("The number of I/O threads currently processing messages"),
      DetailLevelNovice,
      0,
      CounterTypeCounter32);
}

///////////////////////////////////////////////////////////////////////////////
// End of file: PerformanceDataSchema.cpp
///////////////////////////////////////////////////////////////////////////////
