#pragma once
///////////////////////////////////////////////////////////////////////////////
// File: Service.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2004 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/ServiceTools/Service.h"
#include "JetByteTools/ServiceTools/ServiceCallbacks.h"

#include "PerformanceCounters.h"

#include "ServerCommon/SimpleServiceStartupEventHandler.h"

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools
{
   namespace IO
   {
      class CIOPool;
      class CBufferAllocator;
   }

   namespace Socket
   {
      class CSequencedStreamSocketAllocator;
   }
}

class CSocketServer;

///////////////////////////////////////////////////////////////////////////////
// CService
///////////////////////////////////////////////////////////////////////////////

class CService :
   public JetByteTools::Service::CService,
   private JetByteTools::Service::CServiceCallbacks,
   private CSimpleServiceStartupEventHandler
{
   public :

      CService();

      virtual ~CService();

   private :

      // Implement IServiceCallbacks interface

      void OnServiceInstalled() override;
      void OnServiceRemoved() override;

      bool OnInstallPerformanceCounters(
         JetByteTools::Core::_tstring &errorMessage) override;

      bool OnRemovePerformanceCounters(
         JetByteTools::Core::_tstring &errorMessage) override;

      JetByteTools::Service::ServiceTypes::ExitCode InitialiseService(
         JetByteTools::Service::IShutdownService &shutdown,
         JetByteTools::Service::INotifyProgress &notify,
         JetByteTools::Service::IManageServiceNotifications &notificationManager,
         const DWORD argc,
         const PTSTR *ppArgv) override;

      void ContinueService(
         JetByteTools::Service::INotifyProgress &notify) override;

      void PauseService(
         JetByteTools::Service::INotifyProgress &notify) override;

      void StopService(
         JetByteTools::Service::INotifyProgress &notify,
         const JetByteTools::Service::ServiceTypes::StopReason stopReason) override;

      JetByteTools::Service::ServiceTypes::ExitCode UninitialiseService(
         JetByteTools::Service::INotifyProgress &notify) override;

      JetByteTools::Service::ServiceTypes::ExitCode OnException(
         const JetByteTools::Core::CException &e) override;

      JetByteTools::Service::ServiceTypes::ExitCode OnException(
         const JetByteTools::Win32::CSEHException &e) override;

      JetByteTools::Service::ServiceTypes::ExitCode OnException(
         const std::exception &e) override;

      JetByteTools::Service::ServiceTypes::ExitCode OnException() override;

      CPerformanceCounters *m_pCounters;

      JetByteTools::IO::CIOPool *m_pPool;

      JetByteTools::Socket::CSequencedStreamSocketAllocator *m_pSocketAllocator;

      JetByteTools::IO::CBufferAllocator *m_pBufferAllocator;

      CSocketServer *m_pServer;

      bool m_isActive;

      /// No copies do not implement
      CService(const CService &rhs);
      /// No copies do not implement
      CService &operator=(const CService &rhs);
};

///////////////////////////////////////////////////////////////////////////////
// End of file: Service.h
///////////////////////////////////////////////////////////////////////////////
