///////////////////////////////////////////////////////////////////////////////
// File: AsyncFileLog.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2019 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "AsyncFileLog.h"
#include "SimpleServerShutdownHandler.h"

#include "JetByteTools/CoreTools/DebugTrace.h"
#include "JetByteTools/CoreTools/ToString.h"
#include "JetByteTools/CoreTools/Thread.h"
#include "JetByteTools/CoreTools/ErrorCodeToErrorMessage.h"

#include "JetByteTools/CoreTools/ManualResetEvent.h"

#pragma hdrstop

#include <set>

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::IProvideLocalTime;
using JetByteTools::Core::_tstring;
using JetByteTools::Core::OutputEx;
using JetByteTools::Core::ErrorCodeToErrorMessage;

using JetByteTools::Core::CManualResetEvent;

using JetByteTools::IO::IAllocateBuffers;
using JetByteTools::IO::IIOPool;
using JetByteTools::IO::ILimitPendingWrites;
using JetByteTools::IO::CAsyncFileWriter;

///////////////////////////////////////////////////////////////////////////////
// CAsyncFileLog
///////////////////////////////////////////////////////////////////////////////

CAsyncFileLog::CAsyncFileLog(
   const _tstring &filename,
   ILimitPendingWrites &writeLimiter)
   :  JetByteTools::IO::CAsyncFileLog(filename, writeLimiter),
      m_pShutdownHandler(nullptr),
      m_pEvent(nullptr)
{
}

CAsyncFileLog::CAsyncFileLog(
   const _tstring &filename,
   IIOPool &pool,
   IAllocateBuffers &allocator,
   ILimitPendingWrites &writeLimiter)
   :  JetByteTools::IO::CAsyncFileLog(filename, pool, allocator, writeLimiter),
      m_pShutdownHandler(nullptr),
      m_pEvent(nullptr)
{
}

CAsyncFileLog::CAsyncFileLog(
   const _tstring &filename,
   IIOPool &pool,
   IAllocateBuffers &allocator,
   const IProvideLocalTime &timeProvider,
   ILimitPendingWrites &writeLimiter)
   :  JetByteTools::IO::CAsyncFileLog(filename, pool, allocator, timeProvider, writeLimiter),
      m_pShutdownHandler(nullptr),
      m_pEvent(nullptr)
{
}

void CAsyncFileLog::SetShutdownHandler(
   CSimpleServerShutdownHandler &shutdownHandler)
{
   m_pShutdownHandler = &shutdownHandler;
}

void CAsyncFileLog::SetShutdownEvent(
   CManualResetEvent &event)
{
   m_pEvent = &event;
}

void CAsyncFileLog::OnError(
   CAsyncFileWriter * /*pInstance*/,
   const _tstring &where,
   DWORD lastError)
{
   OutputEx(_T("CAsyncFileLog::OnError() - ") + where + _T(" - ") + ErrorCodeToErrorMessage(lastError));
}

void CAsyncFileLog::OnError(
   CAsyncFileWriter * /*pInstance*/,
   const _tstring &where,
   const _tstring &message)
{
   OutputEx(_T("CAsyncFileLog::OnError() - ") + where + _T(" - ") + message);
}

void CAsyncFileLog::OnLastWrite(
   CAsyncFileWriter * /*pInstance*/)
{
}

void CAsyncFileLog::OnPendingWriteLimit(
   CAsyncFileWriter * /*pInstance*/)
{
   MessageBox(nullptr, _T("Too much logging causes tests to fail\nSHUTTING DOWN\n"), _T("CAsyncFileLog::OnPendingWriteLimit()"), MB_OK);

   if (m_pShutdownHandler)
   {
      m_pShutdownHandler->ShutdownNow();
   }

   if (m_pEvent)
   {
      m_pEvent->Set();
   }
}

void CAsyncFileLog::OnThreadPoolThreadDestroyed()
{
   OutputEx(_T("CAsyncFileLog::OnThreadPoolThreadDestroyed()"));
}

void CAsyncFileLog::OnThreadPoolError(
   const _tstring &message)
{
   OutputEx(_T("CAsyncFileLog::OnThreadPoolError() - ") + message);
}

///////////////////////////////////////////////////////////////////////////////
// End of file: AsyncFileLog.cpp
///////////////////////////////////////////////////////////////////////////////
