///////////////////////////////////////////////////////////////////////////////
// File: SimpleServiceStartupEventHandler.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2010 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "SimpleServiceStartupEventHandler.h"

#include "JetByteTools/CoreTools/Exception.h"

#include "JetByteTools/Win32Tools/GlobalName.h"

#pragma hdrstop

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::_tstring;
using JetByteTools::Core::CException;

using JetByteTools::Win32::CManualResetEvent;
using JetByteTools::Win32::CSecurityDescriptorAllowAll;
using JetByteTools::Win32::CSecurityAttributes;
using JetByteTools::Win32::CGlobalName;

///////////////////////////////////////////////////////////////////////////////
// CSimpleServiceStartupEventHandler
///////////////////////////////////////////////////////////////////////////////

CSimpleServiceStartupEventHandler::CSimpleServiceStartupEventHandler()
   :  m_sa(m_sd),
      m_serverStartedEvent(CGlobalName(_T("JetByteToolsServerStarted")), &m_sa),
      m_startupComplete(false)
{
}

CSimpleServiceStartupEventHandler::CSimpleServiceStartupEventHandler(
   const _tstring &startupEventName)
   :  m_sa(m_sd),
      m_serverStartedEvent(CGlobalName(startupEventName), &m_sa),
      m_startupComplete(false)
{
}

void CSimpleServiceStartupEventHandler::StartupComplete()
{
   if (m_startupComplete)
   {
      throw CException(
         _T("CSimpleServiceStartupEventHandler::StartupComplete()"),
         _T("Startup is already complete"));
   }
   m_serverStartedEvent.Set();
}

///////////////////////////////////////////////////////////////////////////////
// End of file: CSimpleServiceStartupEventHandler.cpp
///////////////////////////////////////////////////////////////////////////////
